package Sah::SpecTest;

our $AUTHORITY = 'cpan:PERLANCAR'; # AUTHORITY
our $DATE = '2016-05-12'; # DATE
our $DIST = 'Sah'; # DIST
our $VERSION = '0.9.40'; # VERSION

1;
# ABSTRACT: Just a dummy module so dists can depends on it

__END__

=pod

=encoding UTF-8

=head1 NAME

Sah::SpecTest - Just a dummy module so dists can depends on it

=head1 VERSION

This document describes version 0.9.40 of Sah::SpecTest (from Perl distribution Sah), released on 2016-05-12.

=head1 DESCRIPTION

This is just a dummy module so other Perl distributions can depends on the
spectest (this dist's C<share/spectest/>).

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/Sah>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-Sah>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=Sah>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2016 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
